@@ -11,7 +11,7 @@ use crossterm::{
 };
 use std::cmp::max;
 use std::error::Error;
-use std::time::Duration;
+use std::time::{Duration, Instant};
 use stopwatch::Stopwatch;
 use structopt::StructOpt;
 use tui::{
@@ -55,6 +55,10 @@ struct App {
     word_goal: Option<i64>,
 
     strict_mode: bool,
+
+    last_keystroke: Option<Instant>,
+
+    keystroke_timeout: Option<i64>,
 }
 
 impl App {
@@ -64,6 +68,7 @@ impl App {
         word_goal: Option<i64>,
         backspace_active: bool,
         strict_mode: bool,
+        keystroke_timeout: Option<i64>,
     ) -> App {
         App {
             title,
@@ -74,6 +79,8 @@ impl App {
             word_goal,
             writing_time: Stopwatch::new(),
             strict_mode,
+            last_keystroke: None,
+            keystroke_timeout,
         }
     }
 
@@ -100,7 +107,7 @@ impl App {
                 if duration as i64 >= i {
                     Color::Green
                 } else {
-                    Color::Yellow
+                    Color::Blue
                 }
             }
             None => Color::DarkGray,
@@ -157,6 +164,24 @@ impl App {
             }
         }
     }
+
+    fn get_widget_colors(&self) -> (Color, Color) {
+        match self.input_mode {
+            InputMode::Title => (Color::Blue, Color::DarkGray),
+            InputMode::Writing => match (self.keystroke_timeout, self.last_keystroke) {
+                (Some(timeout), Some(last_keystroke)) => {
+                    if last_keystroke.elapsed().as_secs_f32() > 0.8 * timeout as f32 {
+                        (Color::DarkGray, Color::Red)
+                    } else if last_keystroke.elapsed().as_secs_f32() > 0.5 * timeout as f32 {
+                        (Color::DarkGray, Color::Yellow)
+                    } else {
+                        (Color::DarkGray, Color::Blue)
+                    }
+                }
+                _ => (Color::DarkGray, Color::Blue),
+            },
+        }
+    }
 }
 
 fn get_text_position(text: &str) -> (u16, u16) {
@@ -204,6 +229,10 @@ fn main() -> Result<(), Box<dyn Error>> {
     if let Some(0) = word_goal {
         word_goal = None;
     }
+    let mut keystroke_timeout = settings.get_int("keystroke_timeout").ok();
+    if let Some(0) = keystroke_timeout {
+        keystroke_timeout = None;
+    }
     let strict_mode = settings.get_bool("strict_mode").unwrap_or(true);
 
     // setup terminal
@@ -219,6 +248,7 @@ fn main() -> Result<(), Box<dyn Error>> {
         word_goal,
         backspace_active,
         strict_mode,
+        keystroke_timeout,
     );
     let res = run_app(&mut terminal, &mut app);
 
@@ -273,6 +303,7 @@ fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: &mut App) -> io::Res
                     InputMode::Writing => match key.code {
                         KeyCode::Enter => app.text.push('\n'),
                         KeyCode::Char(c) => {
+                            app.last_keystroke = Some(Instant::now());
                             app.text.push(c);
                         }
                         KeyCode::Backspace => {
@@ -283,6 +314,7 @@ fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: &mut App) -> io::Res
                         KeyCode::Esc => {
                             if app.achieved_goals() || !app.strict_mode {
                                 app.writing_time.stop();
+                                app.last_keystroke = None;
                                 app.input_mode = InputMode::Title;
                             }
                         }
@@ -291,6 +323,14 @@ fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: &mut App) -> io::Res
                 }
             }
         }
+        if let (Some(keystroke_timeout), Some(last_keystroke)) =
+            (app.keystroke_timeout, app.last_keystroke)
+        {
+            if last_keystroke.elapsed().as_secs() > keystroke_timeout as u64 {
+                app.last_keystroke = None;
+                app.text.clear();
+            }
+        };
     }
 }
 
@@ -313,10 +353,7 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &App) {
     let help_message = Paragraph::new(text);
     f.render_widget(help_message, chunks[0]);
 
-    let widget_colors = match app.input_mode {
-        InputMode::Title => (Color::Yellow, Color::DarkGray),
-        InputMode::Writing => (Color::DarkGray, Color::Yellow),
-    };
+    let widget_colors = app.get_widget_colors();
 
     let title = Paragraph::new(app.title.clone())
         .style(Style::default().fg(widget_colors.0))
